/*
 * ValueProvider.hpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _VALUE_PROVIDER_HPP_
#define _VALUE_PROVIDER_HPP_

#include <exception/CommonException.h>
#include <utility>

namespace abstraction {

template < class Type >
class ValueProvider {
	bool m_move;

protected:
	virtual Type & getData ( ) const = 0;

public:
	typedef Type return_type;

	ValueProvider ( ) : m_move ( false ) {
	}

	void setMove ( bool move ) {
		m_move = move;
	}

	Type getValue ( ) const {
		if ( m_move )
			return std::move ( getData ( ) );
		else
			return getData ( );
	}

	virtual bool eval ( ) = 0;

	virtual bool isReady ( ) const = 0;
};

template < class Type >
class ValueProvider < Type & > {
protected:
	virtual Type & getData ( ) const = 0;

public:
	typedef Type return_type;

	void setMove ( bool ) {
		throw exception::CommonException ( "Reference cannot be moved." );
	}

	Type & getValueReference ( ) const {
		return getData ( );
	}

	Type & getValue ( ) const {
		return getData ( );
	}

	virtual bool eval ( ) = 0;

	virtual bool isReady ( ) const = 0;
};

template < class Type >
class ValueProvider < const Type & > {
protected:
	virtual const Type & getConstData ( ) const = 0;

public:
	typedef Type return_type;

	void setMove ( bool ) {
		throw exception::CommonException ( "Const reference cannot be moved." );
	}

	const Type & getConstValueReference ( ) const {
		return getConstData ( );
	}

	const Type & getValue ( ) const{
		return getConstData ( );
	}

	virtual bool eval ( ) = 0;

	virtual bool isReady ( ) const = 0;
};

} /* namespace abstraction */

#endif /* _VALUE_PROVIDER_HPP_ */
